@@ -8,6 +8,7 @@ from .base import (  # noqa
 )
 
 from .access_policy import AccessPolicy, AutoAddObjPermsMixin, AutoDeleteObjPermsMixin  # noqa
+from .acs import AlternateContentSource, AlternateContentSourcePath  # noqa
 from .content import (  # noqa
     Artifact,
     AsciiArmoredDetachedSigningService,
@@ -24,6 +25,7 @@ from .exporter import (  # noqa
     Export,
     ExportedResource,
     Exporter,
+    FilesystemExport,
     FilesystemExporter,
     PulpExport,
     PulpExporter,
@@ -40,6 +42,7 @@ from .publication import (  # noqa
     Publication,
     PublishedArtifact,
     PublishedMetadata,
+    RBACContentGuard,
 )
 from .repository import (  # noqa
     Remote,
@@ -53,10 +56,8 @@ from .status import ContentAppStatus  # noqa
 
 from .task import (  # noqa
     CreatedResource,
-    ReservedResource,
     Task,
     TaskGroup,
-    TaskReservedResource,
     Worker,
 )
 from .upload import (  # noqa
