@@ -3,6 +3,7 @@ Content related Django models.
 """
 from gettext import gettext as _
 
+import datetime
 import json
 import tempfile
 import shutil
@@ -10,6 +11,7 @@ import subprocess
 
 import gnupg
 
+from functools import lru_cache
 from itertools import chain
 
 from django.conf import settings
@@ -163,7 +165,10 @@ class HandleTempFilesMixin:
 
 
 class ArtifactManager(BulkCreateManager):
-    pass
+    def orphaned(self, orphan_protection_time):
+        """Returns set of orphaned artifacts that are ready to be cleaned up."""
+        expiration = now() - datetime.timedelta(minutes=orphan_protection_time)
+        return self.filter(content_memberships__isnull=True, timestamp_of_interest__lt=expiration)
 
 
 class Artifact(HandleTempFilesMixin, BaseModel):
@@ -454,7 +459,16 @@ class PulpTemporaryFile(HandleTempFilesMixin, BaseModel):
 
 
 class ContentManager(BulkCreateManager):
-    pass
+    def orphaned(self, orphan_protection_time, content_pks=None):
+        """Returns set of orphaned content that is ready to be cleaned up."""
+        expiration = now() - datetime.timedelta(minutes=orphan_protection_time)
+        if content_pks:
+            return self.filter(
+                version_memberships__isnull=True,
+                timestamp_of_interest__lt=expiration,
+                pk__in=content_pks,
+            )
+        return self.filter(version_memberships__isnull=True, timestamp_of_interest__lt=expiration)
 
 
 ContentManager = ContentManager.from_queryset(BulkTouchQuerySet)
@@ -473,6 +487,8 @@ class Content(MasterModel, QueryMixin):
         _artifacts (models.ManyToManyField): Artifacts related to Content through ContentArtifact
     """
 
+    PROTECTED_FROM_RECLAIM = True
+
     TYPE = "content"
     repo_key_fields = ()  # Used by pulpcore.plugin.repo_version_utils.remove_duplicates
     upstream_id = models.UUIDField(null=True)  # Used by PulpImport/Export processing
@@ -489,10 +505,23 @@ class Content(MasterModel, QueryMixin):
     @classmethod
     def natural_key_fields(cls):
         """
-        Returns a tuple of the natural key fields which usually equates to unique_together fields
+        Returns a tuple of the natural key fields which usually equates to unique_together fields.
+
+        This can be overwritten in subclasses and should return a tuple of field names.
         """
         return tuple(chain.from_iterable(cls._meta.unique_together))
 
+    @classmethod
+    @lru_cache(typed=True)
+    def _sanitized_natural_key_fields(cls):
+        """
+        This function translates the names of the key fields to their attname.
+
+        In case of foreign keys, this decodes to the corresponding `<...>_id` field preventing
+        extra DB accesses to fetch the related objects.
+        """
+        return tuple(getattr(cls, field).field.attname for field in cls.natural_key_fields())
+
     def natural_key(self):
         """
         Get the model's natural key based on natural_key_fields.
@@ -500,16 +529,13 @@ class Content(MasterModel, QueryMixin):
         Returns:
             tuple: The natural key.
         """
-        return tuple(getattr(self, f) for f in self.natural_key_fields())
+        return tuple(getattr(self, f) for f in self._sanitized_natural_key_fields())
 
     def natural_key_dict(self):
         """
         Get the model's natural key as a dictionary of keys and values.
         """
-        to_return = {}
-        for key in self.natural_key_fields():
-            to_return[key] = getattr(self, key)
-        return to_return
+        return {f: getattr(self, f) for f in self._sanitized_natural_key_fields()}
 
     @staticmethod
     def init_from_artifact_and_relative_path(artifact, relative_path):
@@ -591,6 +617,20 @@ class ContentArtifact(BaseModel, QueryMixin):
         return c_key, a_key
 
 
+class RemoteArtifactQuerySet(models.QuerySet):
+    """QuerySet that provides methods for querying RemoteArtifact."""
+
+    def acs(self):
+        """Return RemoteArtifacts if they belong to an ACS."""
+        return self.filter(remote__alternatecontentsource__isnull=False)
+
+    def order_by_acs(self):
+        """Order RemoteArtifacts returning ones with ACSes first."""
+        return self.annotate(acs_count=models.Count("remote__alternatecontentsource")).order_by(
+            "-acs_count"
+        )
+
+
 class RemoteArtifact(BaseModel, QueryMixin):
     """
     Represents a content artifact that is provided by a remote (external) repository.
@@ -631,7 +671,7 @@ class RemoteArtifact(BaseModel, QueryMixin):
     content_artifact = models.ForeignKey(ContentArtifact, on_delete=models.CASCADE)
     remote = models.ForeignKey("Remote", on_delete=models.CASCADE)
 
-    objects = BulkCreateManager()
+    objects = BulkCreateManager.from_queryset(RemoteArtifactQuerySet)()
 
     def validate_checksums(self):
         """Validate if RemoteArtifact has allowed checksum or no checksum at all."""
