@@ -4,6 +4,7 @@ Repository related Django models.
 from contextlib import suppress
 from gettext import gettext as _
 from os import path
+from collections import defaultdict
 import logging
 
 import django
@@ -11,10 +12,10 @@ from asyncio_throttle import Throttler
 from dynaconf import settings
 from django.core.validators import MinValueValidator
 from django.db import models, transaction
+from django.db.models import Q
 from django.urls import reverse
 from django_lifecycle import AFTER_UPDATE, BEFORE_DELETE, hook
-
-from django.contrib.postgres.fields import JSONField
+from rest_framework.exceptions import APIException
 
 from pulpcore.app.util import batch_qs, get_view_name_for_model
 from pulpcore.constants import ALL_KNOWN_CONTENT_CHECKSUMS
@@ -25,6 +26,7 @@ from pulpcore.cache import Cache
 
 from .base import MasterModel, BaseModel
 from .content import Artifact, Content
+from .fields import EncryptedTextField
 from .task import CreatedResource, Task
 
 
@@ -41,7 +43,7 @@ class Repository(MasterModel):
         description (models.TextField): An optional description.
         next_version (models.PositiveIntegerField): A record of the next version number to be
             created.
-        retained_versions (models.PositiveIntegerField): Number of repo versions to keep
+        retain_repo_versions (models.PositiveIntegerField): Number of repo versions to keep
         user_hidden (models.BooleanField): Whether to expose this repo to users via the API
 
     Relations:
@@ -57,7 +59,7 @@ class Repository(MasterModel):
     name = models.TextField(db_index=True, unique=True)
     description = models.TextField(null=True)
     next_version = models.PositiveIntegerField(default=0)
-    retained_versions = models.PositiveIntegerField(default=None, null=True)
+    retain_repo_versions = models.PositiveIntegerField(default=None, null=True)
     user_hidden = models.BooleanField(default=False)
     content = models.ManyToManyField(
         "Content", through="RepositoryContent", related_name="repositories"
@@ -68,7 +70,7 @@ class Repository(MasterModel):
         verbose_name_plural = "repositories"
 
     def on_new_version(self, version):
-        """Called when new repository versions are created.
+        """Called after a new repository version has been created.
 
         Subclasses are expected to override this to do useful things.
 
@@ -200,11 +202,26 @@ class Repository(MasterModel):
         """
         return Artifact.objects.filter(content__pk__in=version.content)
 
-    @hook(AFTER_UPDATE, when="retained_versions", has_changed=True)
+    @hook(AFTER_UPDATE, when="retain_repo_versions", has_changed=True)
+    def _cleanup_old_versions_hook(self):
+        # Do not attempt to clean up anything, while there is a transaction involving repo versions
+        # still in flight.
+        transaction.on_commit(self.cleanup_old_versions)
+
     def cleanup_old_versions(self):
-        """Cleanup old repository versions based on retained_versions."""
-        if self.retained_versions:
-            for version in self.versions.order_by("-number")[self.retained_versions :]:
+        """Cleanup old repository versions based on retain_repo_versions."""
+        # I am still curious how, but it was reported that this state can happen in day to day
+        # operations but its easy to reproduce manually in the pulpcore shell:
+        # https://github.com/pulp/pulpcore/issues/2268
+        if self.versions.filter(complete=False).exists():
+            raise RuntimeError(
+                _("Attempt to cleanup old versions, while a new version is in flight.")
+            )
+        if self.retain_repo_versions:
+            # Consider only completed versions for cleanup
+            for version in self.versions.complete().order_by("-number")[
+                self.retain_repo_versions :
+            ]:
                 _logger.info(
                     _("Deleting repository version {} due to version retention limit.").format(
                         version
@@ -262,6 +279,7 @@ class Remote(MasterModel):
         connect_timeout (models.FloatField): Value for aiohttp.ClientTimeout.connect
         sock_connect_timeout (models.FloatField): Value for aiohttp.ClientTimeout.sock_connect
         sock_read_timeout (models.FloatField): Value for aiohttp.ClientTimeout.sock_read
+        headers (models.JSONField): Headers set on the aiohttp.ClientSession
         rate_limit (models.IntegerField): Limits total download rate in requests per second.
     """
 
@@ -297,15 +315,15 @@ class Remote(MasterModel):
 
     ca_cert = models.TextField(null=True)
     client_cert = models.TextField(null=True)
-    client_key = models.TextField(null=True)
+    client_key = EncryptedTextField(null=True)
     tls_validation = models.BooleanField(default=True)
 
-    username = models.TextField(null=True)
-    password = models.TextField(null=True)
+    username = EncryptedTextField(null=True)
+    password = EncryptedTextField(null=True)
 
     proxy_url = models.TextField(null=True)
-    proxy_username = models.TextField(null=True)
-    proxy_password = models.TextField(null=True)
+    proxy_username = EncryptedTextField(null=True)
+    proxy_password = EncryptedTextField(null=True)
 
     download_concurrency = models.PositiveIntegerField(
         null=True, validators=[MinValueValidator(1, "Download concurrency must be at least 1")]
@@ -325,7 +343,7 @@ class Remote(MasterModel):
     sock_read_timeout = models.FloatField(
         null=True, validators=[MinValueValidator(0.0, "Timeout must be >= 0")]
     )
-    headers = JSONField(blank=True, null=True)
+    headers = models.JSONField(blank=True, null=True)
     rate_limit = models.IntegerField(null=True)
 
     @property
@@ -496,7 +514,7 @@ class RepositoryVersionQuerySet(models.QuerySet):
     """A queryset that provides repository version filtering methods."""
 
     def complete(self):
-        return self.exclude(complete=False)
+        return self.filter(complete=True)
 
     def with_content(self, content):
         """
@@ -543,13 +561,13 @@ class RepositoryVersion(BaseModel):
         number (models.PositiveIntegerField): A positive integer that uniquely identifies a version
             of a specific repository. Each new version for a repo should have this field set to
             1 + the most recent version.
-        action  (models.TextField): The action that produced the version.
         complete (models.BooleanField): If true, the RepositoryVersion is visible. This field is set
             to true when the task that creates the RepositoryVersion is complete.
 
     Relations:
 
         repository (models.ForeignKey): The associated repository.
+        base_version (models.ForeignKey): The repository version this was created from.
     """
 
     objects = RepositoryVersionQuerySet.as_manager()
@@ -853,9 +871,11 @@ class RepositoryVersion(BaseModel):
         # delete any relationships added in the version being deleted and removed in the next one.
         repo_relations.filter(version_added=self, version_removed=next_version).delete()
 
-        # If the same content is deleted in version, but added back in next_version
-        # set version_removed field in relation to None, and remove relation adding the content
-        # in next_version
+        # If the same content is deleted in version, but added back in next_version then:
+        # - set version_removed field in relation to version_removed of the relation adding
+        #   the content in next version because the content can be removed again after the
+        #   next_version
+        # - and remove relation adding the content in next_version
         content_added = repo_relations.filter(version_added=next_version).values_list("content_id")
 
         # use list() to force the evaluation of the queryset, otherwise queryset is affected
@@ -866,13 +886,26 @@ class RepositoryVersion(BaseModel):
             )
         )
 
-        repo_relations.filter(
-            version_removed=self, content_id__in=content_removed_and_readded
-        ).update(version_removed=None)
-
-        repo_relations.filter(
+        repo_contents_readded_in_next_version = repo_relations.filter(
             version_added=next_version, content_id__in=content_removed_and_readded
-        ).delete()
+        )
+
+        # Since the readded contents can be removed again by any subsequent version after the
+        # next version. Get the mapping of readded contents and their versions removed to use
+        # later. The version removed id will be None if a content is not removed.
+        version_removed_id_content_id_map = defaultdict(list)
+        for readded_repo_content in repo_contents_readded_in_next_version.iterator():
+            version_removed_id_content_id_map[readded_repo_content.version_removed_id].append(
+                readded_repo_content.content_id
+            )
+
+        repo_contents_readded_in_next_version.delete()
+
+        # Update the version removed of the readded contents
+        for version_removed_id, content_ids in version_removed_id_content_id_map.items():
+            repo_relations.filter(version_removed=self, content_id__in=content_ids).update(
+                version_removed_id=version_removed_id
+            )
 
         # "squash" by moving other additions and removals forward to the next version
         repo_relations.filter(version_added=self).update(version_added=next_version)
@@ -892,22 +925,34 @@ class RepositoryVersion(BaseModel):
         Deletion of a complete RepositoryVersion should be done in a RQ Job.
         """
         if self.complete:
+            if self.repository.versions.complete().count() <= 1:
+                raise APIException(_("Attempt to delete the last remaining version."))
             if settings.CACHE_ENABLED:
                 base_paths = self.distribution_set.values_list("base_path", flat=True)
                 if base_paths:
                     Cache().delete(base_key=base_paths)
 
-            repo_relations = RepositoryContent.objects.filter(repository=self.repository)
-            try:
-                next_version = self.next()
-                self._squash(repo_relations, next_version)
+            # Handle the manipulation of the repository version content and its final deletion in
+            # the same transaction.
+            with transaction.atomic():
+                repo_relations = RepositoryContent.objects.filter(
+                    repository=self.repository
+                ).select_for_update()
+                try:
+                    next_version = self.next()
+                    self._squash(repo_relations, next_version)
 
-            except RepositoryVersion.DoesNotExist:
-                # version is the latest version so simply update repo contents
-                # and delete the version
-                repo_relations.filter(version_added=self).delete()
-                repo_relations.filter(version_removed=self).update(version_removed=None)
-            super().delete(**kwargs)
+                except RepositoryVersion.DoesNotExist:
+                    # version is the latest version so simply update repo contents
+                    # and delete the version
+                    repo_relations.filter(version_added=self).delete()
+                    repo_relations.filter(version_removed=self).update(version_removed=None)
+
+                if repo_relations.filter(Q(version_added=self) | Q(version_removed=self)).exists():
+                    raise RuntimeError(
+                        _("Some repo relations of this version were not translated.")
+                    )
+                super().delete(**kwargs)
 
         else:
             with transaction.atomic():
@@ -952,6 +997,10 @@ class RepositoryVersion(BaseModel):
         Returns:
             RepositoryVersion: self
         """
+        if self.complete:
+            raise RuntimeError(
+                _("This Repository version is complete. It cannot be modified further.")
+            )
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
@@ -978,15 +1027,16 @@ class RepositoryVersion(BaseModel):
                     unsupported_types = content_types_seen - content_types_supported
                     if unsupported_types:
                         raise ValueError(
-                            "Saw unsupported content types {}".format(unsupported_types)
+                            _("Saw unsupported content types {}").format(unsupported_types)
                         )
 
                     self.complete = True
                     self.repository.next_version = self.number + 1
-                    self.repository.save()
-                    self.save()
+                    with transaction.atomic():
+                        self.repository.save()
+                        self.save()
+                        self._compute_counts()
                     self.repository.cleanup_old_versions()
-                    self._compute_counts()
                     repository.on_new_version(self)
             except Exception:
                 self.delete()
