@@ -3,7 +3,6 @@ import logging
 import mimetypes
 import os
 import re
-from concurrent.futures import ThreadPoolExecutor
 from gettext import gettext as _
 
 from aiohttp.client_exceptions import ClientResponseError
@@ -11,6 +10,8 @@ from aiohttp.web import FileResponse, StreamResponse, HTTPOk
 from aiohttp.web_exceptions import HTTPForbidden, HTTPFound, HTTPNotFound
 from yarl import URL
 
+from asgiref.sync import sync_to_async
+
 import django
 
 os.environ.setdefault("DJANGO_SETTINGS_MODULE", "pulpcore.app.settings")
@@ -43,14 +44,6 @@ from pulpcore.cache import ContentCache  # noqa: E402
 log = logging.getLogger(__name__)
 
 
-loop = asyncio.get_event_loop()
-# Django ORM is blocking, as are most of our file operations. This means that the
-# standard single-threaded async executor cannot help us. We need to create a separate,
-# thread-based executor to pass our heavy blocking IO work to.
-io_pool_exc = ThreadPoolExecutor(max_workers=2)
-loop.set_default_executor(io_pool_exc)
-
-
 class PathNotResolved(HTTPNotFound):
     """
     The path could not be resolved to a published file.
@@ -102,7 +95,6 @@ class Handler:
     hop_by_hop_headers = [
         "connection",
         "content-encoding",
-        "content-length",
         "keep-alive",
         "public",
         "proxy-authenticate",
@@ -131,7 +123,6 @@ class Handler:
         """
 
         def get_base_paths_blocking():
-            self._reset_db_connection()
             if self.distribution_model is None:
                 base_paths = list(Distribution.objects.values_list("base_path", flat=True))
             else:
@@ -140,7 +131,7 @@ class Handler:
                 )
             return base_paths
 
-        base_paths = await loop.run_in_executor(None, get_base_paths_blocking)
+        base_paths = await sync_to_async(get_base_paths_blocking)()
         directory_list = ["{}/".format(path) for path in base_paths]
         return HTTPOk(headers={"Content-Type": "text/html"}, body=self.render_html(directory_list))
 
@@ -166,7 +157,7 @@ class Handler:
         if index_p1:
             return base_paths[index_p1 - 1]
         else:
-            distro = await loop.run_in_executor(None, cls._match_distribution, path)
+            distro = await sync_to_async(cls._match_distribution)(path)
             return distro.base_path
 
     @classmethod
@@ -183,9 +174,9 @@ class Handler:
         present = await cached.get(guard_key, base_key=base_key)
         if present == b"True" or present is None:
             path = request.match_info["path"]
-            distro = await loop.run_in_executor(None, cls._match_distribution, path)
+            distro = await sync_to_async(cls._match_distribution)(path)
             try:
-                guard = await loop.run_in_executor(None, cls._permit, request, distro)
+                guard = await sync_to_async(cls._permit)(request, distro)
             except HTTPForbidden:
                 guard = True
                 raise
@@ -208,8 +199,6 @@ class Handler:
             :class:`aiohttp.web.StreamResponse` or :class:`aiohttp.web.FileResponse`: The response
                 back to the client.
         """
-        self._reset_db_connection()
-
         path = request.match_info["path"]
         return await self._match_and_stream(path, request)
 
@@ -248,8 +237,6 @@ class Handler:
         Raises:
             PathNotResolved: when not matched.
         """
-        cls._reset_db_connection()
-
         base_paths = cls._base_paths(path)
         if cls.distribution_model is None:
             try:
@@ -405,7 +392,7 @@ class Handler:
             else:
                 raise PathNotResolved(path)
 
-        return await loop.run_in_executor(None, list_directory_blocking)
+        return await sync_to_async(list_directory_blocking)()
 
     async def _match_and_stream(self, path, request):
         """
@@ -441,21 +428,15 @@ class Handler:
                 streamed back to the client.
         """
 
-        def match_distribution_blocking():
-            return self._match_distribution(path)
+        distro = await sync_to_async(self._match_distribution)(path)
 
-        distro = await loop.run_in_executor(None, match_distribution_blocking)
-
-        def check_permit_blocking():
-            self._permit(request, distro)
-
-        await loop.run_in_executor(None, check_permit_blocking)
+        await sync_to_async(self._permit)(request, distro)
 
         rel_path = path.lstrip("/")
         rel_path = rel_path[len(distro.base_path) :]
         rel_path = rel_path.lstrip("/")
 
-        content_handler_result = distro.content_handler(rel_path)
+        content_handler_result = await sync_to_async(distro.content_handler)(rel_path)
         if content_handler_result is not None:
             return content_handler_result
 
@@ -488,23 +469,24 @@ class Handler:
                 if not repo_version:
                     repo_version = repository.latest_version()
 
-            await loop.run_in_executor(None, get_latest_publication_or_version_blocking)
+            await sync_to_async(get_latest_publication_or_version_blocking)()
 
         if publication:
             if rel_path == "" or rel_path[-1] == "/":
                 try:
                     index_path = "{}index.html".format(rel_path)
 
-                    def get_published_artifact_blocking():
-                        publication.published_artifact.get(relative_path=index_path)
-
-                    await loop.run_in_executor(None, get_published_artifact_blocking)
+                    await sync_to_async(publication.published_artifact.get)(
+                        relative_path=index_path
+                    )
 
                     rel_path = index_path
                     headers = self.response_headers(rel_path)
                 except ObjectDoesNotExist:
                     dir_list = await self.list_directory(None, publication, rel_path)
-                    dir_list.update(distro.content_handler_list_directory(rel_path))
+                    dir_list.update(
+                        await sync_to_async(distro.content_handler_list_directory)(rel_path)
+                    )
                     return HTTPOk(
                         headers={"Content-Type": "text/html"}, body=self.render_html(dir_list)
                     )
@@ -522,7 +504,7 @@ class Handler:
                         .content_artifact
                     )
 
-                ca = await loop.run_in_executor(None, get_contentartifact_blocking)
+                ca = await sync_to_async(get_contentartifact_blocking)()
             except ObjectDoesNotExist:
                 pass
             else:
@@ -544,7 +526,7 @@ class Handler:
                         )
                         return ca
 
-                    ca = await loop.run_in_executor(None, get_contentartifact_blocking)
+                    ca = await sync_to_async(get_contentartifact_blocking)()
                 except MultipleObjectsReturned:
                     log.error(
                         _("Multiple (pass-through) matches for {b}/{p}"),
@@ -570,14 +552,14 @@ class Handler:
                         content__in=repo_version.content, relative_path=index_path
                     ).exists()
 
-                contentartifact_exists = await loop.run_in_executor(
-                    None, contentartifact_exists_blocking
-                )
+                contentartifact_exists = await sync_to_async(contentartifact_exists_blocking)()
                 if contentartifact_exists:
                     rel_path = index_path
                 else:
                     dir_list = await self.list_directory(repo_version, None, rel_path)
-                    dir_list.update(distro.content_handler_list_directory(rel_path))
+                    dir_list.update(
+                        await sync_to_async(distro.content_handler_list_directory)(rel_path)
+                    )
                     return HTTPOk(
                         headers={"Content-Type": "text/html"}, body=self.render_html(dir_list)
                     )
@@ -590,7 +572,7 @@ class Handler:
                     )
                     return ca
 
-                ca = await loop.run_in_executor(None, get_contentartifact_blocking)
+                ca = await sync_to_async(get_contentartifact_blocking)()
             except MultipleObjectsReturned:
                 log.error(
                     _("Multiple (pass-through) matches for {b}/{p}"),
@@ -612,7 +594,7 @@ class Handler:
             def cast_remote_blocking():
                 return distro.remote.cast()
 
-            remote = await loop.run_in_executor(None, cast_remote_blocking)
+            remote = await sync_to_async(cast_remote_blocking)()
 
             try:
                 url = remote.get_remote_artifact_url(rel_path)
@@ -625,7 +607,7 @@ class Handler:
                     ).get(remote=remote, url=url)
                     return ra
 
-                ra = await loop.run_in_executor(None, get_remote_artifact_blocking)
+                ra = await sync_to_async(get_remote_artifact_blocking)()
                 ca = ra.content_artifact
                 if ca.artifact:
                     return await self._serve_content_artifact(ca, headers)
@@ -666,9 +648,9 @@ class Handler:
         """
 
         def get_remote_artifacts_blocking():
-            return list(content_artifact.remoteartifact_set.all())
+            return list(content_artifact.remoteartifact_set.select_related("remote").order_by_acs())
 
-        remote_artifacts = await loop.run_in_executor(None, get_remote_artifacts_blocking)
+        remote_artifacts = await sync_to_async(get_remote_artifacts_blocking)()
         for remote_artifact in remote_artifacts:
             try:
                 response = await self._stream_remote_artifact(request, response, remote_artifact)
@@ -811,7 +793,7 @@ class Handler:
         Args:
             request(:class:`~aiohttp.web.Request`): The request to prepare a response for.
             response (:class:`~aiohttp.web.StreamResponse`): The response to stream data to.
-            content_artifact (:class:`~pulpcore.plugin.models.ContentArtifact`): The ContentArtifact
+            remote_artifact (:class:`~pulpcore.plugin.models.RemoteArtifact`): The RemoteArtifact
                 to fetch and then stream back to the client
 
         Raises:
@@ -822,10 +804,7 @@ class Handler:
 
         """
 
-        def cast_remote_blocking():
-            return remote_artifact.remote.cast()
-
-        remote = await loop.run_in_executor(None, cast_remote_blocking)
+        remote = await sync_to_async(remote_artifact.remote.cast)()
 
         range_start, range_stop = request.http_range.start, request.http_range.stop
         if range_start or range_stop:
@@ -834,9 +813,10 @@ class Handler:
         async def handle_response_headers(headers):
             for name, value in headers.items():
                 lower_name = name.lower()
-                if lower_name not in self.hop_by_hop_headers:
-                    response.headers[name] = value
-                elif response.status == 206 and lower_name == "content-length":
+                if lower_name in self.hop_by_hop_headers:
+                    continue
+
+                if response.status == 206 and lower_name == "content-length":
                     content_length = int(value)
                     start = 0 if range_start is None else range_start
                     stop = content_length if range_stop is None else range_stop
@@ -849,6 +829,9 @@ class Handler:
                     response.headers["Content-Range"] = "bytes {0}-{1}/{2}".format(
                         start, stop - 1, content_length
                     )
+                    continue
+
+                response.headers[name] = value
             await response.prepare(request)
 
         data_size_handled = 0
@@ -885,11 +868,9 @@ class Handler:
         download_result = await downloader.run()
 
         if remote.policy != Remote.STREAMED:
-
-            def save_artifact_blocking():
-                self._save_artifact(download_result, remote_artifact)
-
-            await asyncio.shield(loop.run_in_executor(None, save_artifact_blocking))
+            await asyncio.shield(
+                sync_to_async(self._save_artifact)(download_result, remote_artifact)
+            )
         await response.write_eof()
 
         if response.status == 404:
