@@ -2,6 +2,8 @@
 # Any models defined in the pulpcore.plugin namespace should probably be proxy models.
 
 from pulpcore.app.models import (  # noqa
+    AlternateContentSource,
+    AlternateContentSourcePath,
     AccessPolicy,
     AutoAddObjPermsMixin,
     AutoDeleteObjPermsMixin,
