@@ -1,5 +1,7 @@
 # Import Serializers in platform that are potentially useful to plugin writers
 from pulpcore.app.serializers import (  # noqa
+    AlternateContentSourceSerializer,
+    AlternateContentSourcePathSerializer,
     ArtifactSerializer,
     AsyncOperationResponseSerializer,
     ContentChecksumSerializer,
@@ -8,7 +10,6 @@ from pulpcore.app.serializers import (  # noqa
     DistributionSerializer,
     ExporterSerializer,
     ExportSerializer,
-    FilesystemExporterSerializer,
     IdentityField,
     ImporterSerializer,
     ImportSerializer,
@@ -18,7 +19,6 @@ from pulpcore.app.serializers import (  # noqa
     NestedRelatedField,
     NoArtifactContentSerializer,
     ProgressReportSerializer,
-    PublicationExportSerializer,
     PublicationSerializer,
     RelatedField,
     RemoteSerializer,
@@ -27,6 +27,7 @@ from pulpcore.app.serializers import (  # noqa
     RepositoryVersionRelatedField,
     SingleArtifactContentSerializer,
     SingleContentArtifactField,
+    TaskGroupOperationResponseSerializer,
     ValidateFieldsMixin,
     validate_unknown_fields,
 )
