@@ -1,9 +1,10 @@
 # Allow plugin viewsets to return 202s
-from pulpcore.app.response import OperationPostponedResponse  # noqa
+from pulpcore.app.response import OperationPostponedResponse, TaskGroupOperationResponse  # noqa
 
 # Import Viewsets in platform that are potentially useful to plugin writers
 from pulpcore.app.viewsets import (  # noqa
     AsyncUpdateMixin,
+    AlternateContentSourceViewSet,
     BaseFilterSet,
     ContentFilter,
     ContentGuardFilter,
@@ -18,7 +19,6 @@ from pulpcore.app.viewsets import (  # noqa
     ImporterViewSet,
     NamedModelViewSet,
     NAME_FILTER_OPTIONS,
-    NewDistributionFilter,
     PublicationFilter,
     PublicationViewSet,
     ReadOnlyContentViewSet,
